#ifndef DATA_H
#define DATA_H

#include <sys/types.h>
#include <regex.h>

/* String lengths of account settings. */
#define SERVER_LEN		(635 + 1)	/* Section 2.5 RFC 1123. */
#define USERNAME_LEN		128
#define PASSWORD_LEN		128

#ifdef SSL_TLS
/* Secure Socket Layer protocols. */
#define SSL_DISABLED		0
#define SSL_SSL_V2		1
#define SSL_SSL_V3		2
#define SSL_TLS_V1		3
#endif

/* String length of keys (aliases). */
#define KEY_LEN			32

/* String length of action's arguments. */
#define ARGS_LEN		256

/* String length of mailbox's name. */
#define MBOX_NAME_LEN		128

/* String length of mask's body. */
#define MASK_BODY_LEN		256

/* Maximum filters assigned to a mailbox. */
#define MBOX_FILTERS_MAX	64

/* Maximum mailboxes contained in a mailbox-group. */
#define MBOXGRP_MBOXES_MAX	64

/*
 * Mode for masks that comprise a filter. So if a filter is of AND type then
 * as default masks are ANDed.
 */
#define FILTER_MODE_OR		1
#define FILTER_MODE_AND		2

/* Action to do in case of filter matching. */
#define FILTER_ACTION_DELETE	1
#define FILTER_ACTION_COPY	2
#define FILTER_ACTION_MOVE	3
#define FILTER_ACTION_LIST	4

/* Type of mask. This overrides the default filter mode (AND/OR). */
#define MASK_TYPE_OR		1
#define MASK_TYPE_AND		2


/*
 * Find last node of linked list and append at the end of the list the
 * supplied node.
 */
#define APPEND_LINKED_LIST(f, n, p, a) \
    a = &f; \
    p = f; \
    while (p) { \
	a = &(p->next); \
	p = p->next; \
	} \
    *a = n;

/*
 * Find, based on the key of the supplied node, the apropriate position for it
 * in the tree, and then insert it there.
 */
#define INSERT_TREE(r, n, p, i, c) \
    i = &r; \
    p = r; \
    while (p) { \
	c = strncmp(n->key, p->key, KEY_LEN); \
	if (c < 0) { \
	    i = &(p->left); \
	    p = p->left; \
        } else if (c > 0) { \
	    i = &(p->right); \
	    p = p->right; \
	} else \
	    return; \
	} \
    *i = n;

/*
 * Find, based on the supplied key, the position of a node, and return a
 * pointer directing to that node.
 */
#define FIND_TREE(r, k, p, c) \
    p = r; \
    while (p) { \
	c = strncmp(k, p->key, KEY_LEN); \
	if (c < 0) \
	    p = p->left; \
	else if (c > 0) \
	    p = p->right; \
	else \
	    return p; \
	} \
    return NULL;

#define create_node(A)		xmalloc(A)


/* Account. */
typedef struct account_t {
    char server[SERVER_LEN];	/* Hostname of mail server. */
    unsigned short int port;	/* Port to connect. */

    char username[USERNAME_LEN];/* Username. */
    char password[PASSWORD_LEN];/* Password. */
#ifdef SSL_TLS
    unsigned int ssl;		/* Secure Socket Layer support. */
#endif
    struct mbox_t *mboxes;	/* Mailboxes. */
    struct account_t *next;	/* Next node of linked list. */
} account_t;

/* Mailbox. */
typedef struct mbox_t {
    char name[MBOX_NAME_LEN];	/* Name of mailbox. */
    struct filter_t *filters[MBOX_FILTERS_MAX];	/* Filters to be applied. */
    struct mbox_t *next;	/* Next node of linked list. */
} mbox_t;

/* Group of mailboxes. */
typedef struct mboxgrp_t {
    char key[KEY_LEN];		/* Alias of mailbox group. */
    struct mbox_t *mboxes[MBOXGRP_MBOXES_MAX];	/* Mailboxes of group. */
    struct mboxgrp_t *left, *right;	/* Left/right nodes of tree. */
} mboxgrp_t;

/* Filter. */
typedef struct filter_t {
    char key[KEY_LEN];		/* Alias of filter. */
    unsigned int mode;		/* AND/OR mode. */

    struct {			/* What to do on filter match. */
	unsigned int type;	/* Action. */
	char destmbox[MBOX_NAME_LEN];	/* Destination mailbox. */
	char args[ARGS_LEN];	/* Action's arguments. */
    } action;

    struct mask_t *masks;	/* Masks comprising the filter. */
    unsigned int masknum;	/* Total number of masks. */
    unsigned int masklen;	/* Total length of body of masks. */

    struct filter_t *left, *right;	/* Left/right node of tree. */
} filter_t;

/* Mask. */
typedef struct mask_t {
    char body[MASK_BODY_LEN];	/* Body of mask. */
    unsigned int type;		/* AND/OR type. */
    struct mask_t *next;	/* Next node of linked list. */
} mask_t;


/*	data.c		*/
void init_account(account_t * node);
void append_account(account_t * node);
int set_account(char *line, regmatch_t * match);

void init_mboxgrp(mboxgrp_t * node);
void ins_mboxgrp(mboxgrp_t * node);
int set_mboxgrp(char *line, regmatch_t * match);
void process_mboxgrp(mboxgrp_t * node, char *mboxs);
mboxgrp_t *find_mboxgrp(char *key);

void init_mbox(mbox_t * node);
void append_mbox(mbox_t * node);
mbox_t *set_mbox(char *name);

void init_filter(filter_t * node);
void ins_filter(filter_t * node);
int set_filter(char *line, regmatch_t * match);
filter_t *find_filter(char *key);
int set_action(char *line, regmatch_t * match);

void init_mask(mask_t * node);
void append_mask(mask_t * node);
int set_mask(char *line, regmatch_t * match);
void convert_date(mask_t * node);

int set_job(char *line, regmatch_t * match);
void link_mbox_filter(filter_t * cf, mboxgrp_t * cg);

void destroy_data(void);
void destroy_mboxgrp(mboxgrp_t * node);
void overwrite_passwords(void);

void string_upper(char *str, size_t size);
int string_decode(char *str);

#endif				/* DATA_H */
